//============================================================================
//  aPCI-8102 for PCI Device Driver Diagnosis program
//  apci8102_diag.c
//  Author   K. Fujimaki(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date     2001/07/16
//  Rev.     1.0.0
//  Note
//============================================================================

#include <stdio.h>	// used printf,perror...
#include <unistd.h>	// used read
#include <sys/ioctl.h>
#include <term.h>
#include <string.h>	// used strerror
#include <errno.h>	// used errno

#include "apci8102ctl.h"


// input of command from user
static char line[256];
// port name characters
static char* pname = "ABCD";

//--- Ǵؿ ----------------------------------------------

static void diag01_create(void);
static void diag02_close(void);
static void diag03_get_dirs(void);
static void diag04_set_dirs(void);
static void diag05_inp(void);
static void diag06_outp(void);

static void diag07_test_inp(void);
static void diag08_test_outp(void);

static int	  enter_log_slot(void);
static struct termio init_termio(void);
static void   term_termio(struct termio tty);
static char*  itobina(int data, int size, char* dst);


//--- ؿơ֥ ------------------------------------------

static void (*pdiag_func[])(void) =
{
	diag01_create,
	diag02_close,
	diag03_get_dirs,
	diag04_set_dirs,
	diag05_inp,
	diag06_outp,
	diag07_test_inp,
	diag08_test_outp,
};

int main(int argc, char *argv[])
{
	int cmd, i;

	printf("aPCI-8102 diagnostic utility.\n");

	do
	{
		printf("\n");
		printf("aPCI-8102 Test main menu\n");
		printf("------------------------\n");
		printf(" Function test          \n");	// ǽƥ
		printf("------------------------\n");
		printf(" 1. apci8102_create(int log_slot)                                \n");
		printf(" 2. apci8102_close(int log_slot)                                 \n");
		printf(" 3. apci8102_get_dirs(int log_slot, int port, unsigned char* pdirs)   \n");
		printf(" 4. apci8102_set_dirs(int log_slot, int port, unsigned char dirs)   \n");
		printf(" 5. apci8102_inp(int log_slot, int port, unsigned char* pdata)   \n");
		printf(" 6. apci8102_outp(int log_slot, int port, unsigned char data)    \n");
		printf("------------------------\n");
		printf(" Application test       \n");	// ѥƥ
		printf("------------------------\n");
		printf(" 7. Inport                                                       \n");
		printf(" 8. Outport                                                      \n");
		printf("------------------------\n");
		printf("99. Exit (All devices are closed)\n");
		printf("Enter option: ");

		cmd = 0;
		fgets(line, sizeof(line), stdin);
		sscanf(line, "%d", &cmd);

		if (cmd > 0 && cmd <= sizeof(pdiag_func) / sizeof(void*))
			pdiag_func[cmd - 1]();

		if(cmd != 99)
		{
			printf("--- Please enter key push! ---");
			fgets(line, sizeof(line), stdin);
		}
	}
	while (cmd != 99);

	// ƤΥǥХ򥯥
	for (i = 0; i < APCI8102_MAX_DEVICES; i++)
		apci8102_close(i);

	return 0;
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥץ					       		    */
/*--------------------------------------------------------------------------*/
static void diag01_create(void)
{
	int log_slot;

	printf("--- apci8102_create(int log_slot)\n");

	// Get LogSlot
	printf("Enter log_slot(0-15 or -1): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	// Call Control Function
	log_slot = apci8102_create(log_slot);

	// result
	if (log_slot < 0)
		printf("Rslt: %d\n", log_slot);
	else
		printf("log_slot: %d\n", log_slot);
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥ							    */
/*--------------------------------------------------------------------------*/
static void diag02_close(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8102_close(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8102_close(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*	ǥ쥯							*/
/*--------------------------------------------------------------------------*/
static void diag03_get_dirs(void)
{
	int log_slot;
	unsigned char dirs;
	int rslt;

	printf("--- apci8102_get_dirs(int log_slot, int port, unsigned char* pdirs)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8102_get_dirs(log_slot, &dirs);

	printf("Rslt: %d\n", rslt);
	printf("direction bits(hex): %x\n", dirs);
}

/*--------------------------------------------------------------------------*/
/*  ǥ쥯							    */
/*--------------------------------------------------------------------------*/
static void diag04_set_dirs(void)
{
	int log_slot;
	unsigned int dirs;
	int rslt;

	printf("--- apci8102_set_dirs(int log_slot, int port, unsigned char dirs)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Direction bits
	printf("Enter direction bits(hex): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%x", &dirs);

	// Call Control Function
	rslt = apci8102_set_dirs(log_slot, (unsigned char)dirs);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡ						    */
/*--------------------------------------------------------------------------*/
static void diag05_inp(void)
{
	int log_slot, port;
	unsigned char data;
	int rslt;

	printf("--- apci8102_inp(int log_slot, int port, unsigned char* pdata)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Port
	printf("Enter port: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &port);

	// Call Control Function
	rslt = apci8102_inp(log_slot, port, &data);

	printf("Rslt: %d\n", rslt);
	printf("data(hex): %x\n", data);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤإǡ						    */
/*--------------------------------------------------------------------------*/
static void diag06_outp(void)
{
	int log_slot, port;
	unsigned int data;
	int rslt;

	printf("--- apci8102_outp(int log_slot, int port, unsigned char data)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Port
	printf("Enter port: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &port);

	// Get Data
	printf("Enter data(hex): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%x", &data);

	// Call Control Function
	rslt = apci8102_outp(log_slot, port, (unsigned char)data);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡϢ³					    */
/*--------------------------------------------------------------------------*/
static void diag07_test_inp(void)
{
	int    log_slot, i;
	char   c, n;
	struct termio tty_save;
	char   data, szbuf[16];
	int    rslt;

	printf("--- The continuous test of Inport\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Set directions
	rslt = apci8102_set_dirs(log_slot, 0x0f);
	if (rslt < 0)
	{
		printf(" Rslt: %d\n", rslt);
	}
	else
	{
		printf("\nA completion key is \"ESC\".\n\n");

		for (i = 0; i < APCI8102_MAX_PORTS; i++)
			printf(" PORT%c IN  ", pname[i]);
		printf("\n");

		for (i = 0; i < APCI8102_MAX_PORTS; i++)
			printf("-----------");
		printf("\n");

		tty_save = init_termio();

		for (;;)
		{
			c = '\0';		// ʤ硢ƤѤǤޤ
			n = read(0, &c, 1);
			if (n == -1)
			{
				perror("read");
				break;
			}
			else if (c == 0x1B)	// ESC key
				break;

			// InPort 
			for (i = 0; i < APCI8102_MAX_PORTS; i++)
			{
				rslt = apci8102_inp(log_slot, i, &data);
				if (rslt < 0)
				{
					printf(" Rslt: %d ", rslt);
					continue;
				}

				printf(" %s  ", itobina(data, sizeof(data), szbuf));
			}

			printf("\r");
			fflush(stdout);
		}

		// ᤹
		term_termio(tty_save);

		printf("\n");
	}
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤإǡϢ³					    */
/*--------------------------------------------------------------------------*/
static void diag08_test_outp(void)
{
	int	  log_slot, i;
	char	  c, n;
	struct	  termio tty_save;
	char	  data, szbuf[16];
	int	  rslt;

	printf("--- The continuous test of Outport\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Set directions
	rslt = apci8102_set_dirs(log_slot, 0x0);
	if (rslt < 0)
	{
		printf(" Rslt: %d ", rslt);
	}
	else
	{
		printf("\nA completion key is \"ESC\".\n\n");

		for (i = 0; i < APCI8102_MAX_PORTS; i++)
			printf(" PORT%c OUT ", pname[i]);
		printf("\n");
		for (i = 0; i < APCI8102_MAX_PORTS; i++)
			printf("-----------");
		printf("\n");

		tty_save = init_termio();

		data = 0x00;

		for (;;)
		{
			c = '\0';		// ʤ硢ƤѤǤޤ
			n = read(0, &c, 1);
			if (n == -1)
			{
				perror("read");
				break;
			}
			else if (c == 0x1B)	// ESC key
				break;

			// OutPort 
			for (i = 0; i < APCI8102_MAX_PORTS; i++)
			{
				rslt = apci8102_outp(log_slot, i, data++);
				if (rslt < 0)
				{
					printf(" Rslt: %d ", rslt);
					continue;
				}

				printf(" %s  ", itobina(data, sizeof(data), szbuf));
			}

			data++; // Dummy increment

			printf("\r");
			fflush(stdout);
		}

		// ᤹
		term_termio(tty_save);

		printf("\n");
	}
	// Set directions
	rslt = apci8102_set_dirs(log_slot, 0x0f);
}

/*--------------------------------------------------------------------------*/
/*  åȤ							    */
/*--------------------------------------------------------------------------*/
static int enter_log_slot(void)
{
	int log_slot;

	printf("Enter log_slot: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	return log_slot;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७						    */
/*--------------------------------------------------------------------------*/
static struct termio init_termio(void)
{
	struct termio tty, tty_save;

	// ߤ¸
	ioctl(0, TCGETA, &tty);

	tty_save = tty;

	// ü
	tty.c_lflag &= ~(ECHO | ICANON);
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 2;
	ioctl(0, TCSETAF, &tty);

	return tty_save;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७ϲ						    */
/*--------------------------------------------------------------------------*/
static void term_termio(struct termio tty)
{
	// ᤹
	ioctl (0, TCSETAF, &tty);
}

/*--------------------------------------------------------------------------*/
/*   ---> 2 ʿʸ						    */
/*--------------------------------------------------------------------------*/
static char* itobina(int data, int size, char* dst)
{
	int i;
	int bits = size * 8;
	for (i = 0; i < bits; i++, data >>= 1)
		dst[bits - 1 - i] = '0' + (data & 0x1);
	dst[bits] = '\0';

	return dst;
}

